package com.mangst.io;

/*
 * Copyright 2009 Mike Angstadt
 * mike.angstadt@gmail.com
 * http://www.mangst.com/
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;

/**
 * Reads a stream one line at a time, skipping commented and empty (or whitespace-only) lines.
 * @author mangst
 *
 */
public class SpecialBufferedReader extends BufferedReader {

	public SpecialBufferedReader(Reader in) {
		super(in);
	}

	/**
	 * Reads in the next line.  Ignores empty lines, lines only with whitespace, and lines that start with '#'.
	 * @return the next line or null if EOF
	 */
	@Override
	public String readLine() throws IOException {
		String s = null;
		boolean repeat = true;
		while (repeat) {
			s = super.readLine();
			if (s == null) {
				repeat = false;
			} else {
				String trimmed = s.trim();
				if (trimmed.length() != 0 && trimmed.charAt(0) != '#') {
					repeat = false;
				}
			}
		}
		return s;
	}

}
