package com.mangst.sudoku;

/*
 * Copyright 2009 Mike Angstadt
 * mike.angstadt@gmail.com
 * http://www.mangst.com/
 */

import java.util.HashSet;
import java.util.Set;

public class Cell {
	/**
	 * Value of an empty cell.
	 */
	private static final int EMPTY = 0;
	
	/**
	 * Cell's value when filled.
	 */
	public int value;
	
	/**
	 * Index of the row, col, square the cell belongs to.
	 */
	public int row, col, square;
	
	/**
	 * If the cell is empty, contains all possible values the cell could have.
	 */
	public Set<Integer> poss = new HashSet<Integer>();

	/**
	 * Creates a new cell.
	 * @param value value of the cell
	 */
	public Cell(int value) {
		this.value = value;
	}

	/**
	 * Creates an empty cell.
	 */
	public Cell() {
		this(EMPTY);
	}

	/**
	 * Copy constructor.
	 * @param cell cell to copy
	 */
	public Cell(Cell cell) {
		this(cell.value);
		this.row = cell.row;
		this.col = cell.col;
		this.square = cell.square;
	}

	/**
	 * Determines if the cell is empty or not.
	 * @return true if empty, false if it has a value
	 */
	public boolean isEmpty() {
		return value == EMPTY;
	}

	@Override
	public String toString() {
		if (isEmpty()) {
			return ".";
		}
		return value + "";
	}
}
