package com.mangst.sudoku;

/*
 * Copyright 2009 Mike Angstadt
 * mike.angstadt@gmail.com
 * http://www.mangst.com/
 */

import java.io.StringReader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.Assert;
import org.junit.Test;

public class BoardTest {
	@Test
	public void testBadInput() throws Exception {
		Set<String> inputs = new HashSet<String>();

		//BOARD INPUT====
		//board cannot be of size <= 0
		inputs.add("0\n");
		//board cannot be of size <= 0
		inputs.add("-3\n");
		//first row has only 8 cols
		inputs.add("9\n. . 8 . . . . 4\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .");
		//cell (0,0) has invalid number
		inputs.add("9\n28 . 8 . . . . 4 7\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .");
		//cell (0,0) has invalid number
		inputs.add("9\n-5 . 8 . . . . 4 7\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .");
		//cells (0,0) and (0,1) are the same
		inputs.add("9\n1 1 8 . . . . 4 7\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .");
		//cells (0,0) and (1,0) are the same
		inputs.add("9\n1 . 8 . . . . 4 7\n1 . . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .");
		//cells (0,0) and (1,1) are the same
		inputs.add("9\n1 . 8 . . . . 4 7\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .");

		//AREA INPUT====
		//first row has only 8 cols
		inputs.add("9\n. . 8 . . . . 4 7\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .\n1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9");
		//cell (0,0) has invalid number
		inputs.add("9\n. . 8 . . . . 4 7\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .\n-44 1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9");
		//cell (0,0) has invalid number
		inputs.add("9\n. . 8 . . . . 4 7\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .\n23 1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9");
		//too many 1s
		inputs.add("9\n. . 8 . . . . 4 7\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .\n1 1 1 1 1 1 1 1 1\n1 1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9");
		//incomplete board info
		inputs.add("5\n1 2 3 4 5\n1 2 3 4 5\n1 2 3 4 5\n");
		//non-perfect-square board with invalid input and without square info
		inputs.add("5\n1 2 3 4 5\n1 2 3 4 5\n1 2 3 4 5\n1 2 3 4 5\n1 2 3 4 5");

		for (String s : inputs) {
			try {
				new Board(new StringReader(s));
				Assert.fail();
			} catch (BoardFormatException e) {
				//should throw exception
			}
		}
	}

	@Test
	public void testGoodInput() throws Exception {
		String s;
		List<String> inputs = new ArrayList<String>();
		List<Integer[][]> values = new ArrayList<Integer[][]>();
		List<Integer[][]> squares = new ArrayList<Integer[][]>();
		
		//test case 1 - 9x9 board
		s = "9\na b 8 x - + . 4 7\n"; //empty cells can be any non-numeric char
		s += ".  1 \t. . 3 8\t  2\t\t. .\n"; //cells can be seperated by any amount of tabs or spaces
		s += ". 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .\n1 1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9";
		inputs.add(s);
		values.add(new Integer[][]{{0,0,8,0,0,0,0,4,7},{0,1,0,0,3,8,2,0,0},{0,5,0,1,0,0,0,0,3},{2,0,0,0,0,7,8,9,0},{0,0,0,0,1,0,0,0,0},{0,9,6,3,0,0,0,0,2},{8,0,0,0,0,2,0,6,0},{0,0,9,6,5,0,0,1,0},{3,6,0,0,0,0,4,0,0}});
		squares.add(new Integer[][]{{1,1,1,2,2,2,3,3,3},{1,1,1,2,2,2,3,3,3},{1,1,1,2,2,2,3,3,3},{4,4,4,5,5,5,6,6,6},{4,4,4,5,5,5,6,6,6},{4,4,4,5,5,5,6,6,6},{7,7,7,8,8,8,9,9,9},{7,7,7,8,8,8,9,9,9},{7,7,7,8,8,8,9,9,9}});
		
		//test case 2 - 4x4 board without square inputs
		inputs.add("4\n. 2 . 4\n3 4 . .\n. 1 . 3\n4 . . 1");
		values.add(new Integer[][]{{0,2,0,4},{3,4,0,0},{0,1,0,3},{4,0,0,1}});
		squares.add(new Integer[][]{{1,1,2,2},{1,1,2,2},{3,3,4,4},{3,3,4,4}});
		
		for (int i = 0; i < inputs.size(); ++i){
			String input = inputs.get(i);
			Integer square[][] = squares.get(i);
			Integer value[][] = values.get(i);
			Board b = new Board(new StringReader(input));
			for (int row = 0; row < b.size; ++row){
				for (int col = 0; col < b.size; ++col){
					Cell c = b.getCell(row, col);
					Assert.assertTrue(c.row == row);
					Assert.assertTrue(c.col == col);
					Assert.assertTrue(c.square == square[row][col]-1);
					if (value[row][col] == 0){
						Assert.assertTrue(c.isEmpty());
					} else {
						Assert.assertTrue(!c.isEmpty());
						Assert.assertTrue(c.value == value[row][col]);
					}
				}
			}
		}
	}

	@Test
	public void testBadGetCell() throws Exception {
		String s = "9\n. . 8 . . . . 4 7\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .";
		Board b = new Board(new StringReader(s));

		int inputs[][] = { { -1, 0 }, { 0, -1 }, { 9, 0 }, { 0, 9 }, { 9, 9 } };
		for (int input[] : inputs) {
			try {
				b.getCell(input[0], input[1]);
				Assert.fail();
			} catch (IndexOutOfBoundsException e) {
				//should throw exception
			}
		}
	}

	@Test
	public void testToString() throws Exception {
		String s = "9\n. . 8 . . . . 4 7\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .";
		Board b = new Board(new StringReader(s));
		Assert.assertTrue(b.toString().equals(". . 8 . . . . 4 7\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . ."));
	}

	@Test
	public void testSolve() throws Exception {
		List<String> inputs = new ArrayList<String>();
		List<Integer[][]> solutions = new ArrayList<Integer[][]>();

		//easy
		inputs.add("9\n. . 8 . . . . 4 7\n. 1 . . 3 8 2 . .\n. 5 . 1 . . . . 3\n2 . . . . 7 8 9 .\n. . . . 1 . . . .\n. 9 6 3 . . . . 2\n8 . . . . 2 . 6 .\n. . 9 6 5 . . 1 .\n3 6 . . . . 4 . .\n1 1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9");
		solutions.add(new Integer[][] { { 6, 3, 8, 9, 2, 5, 1, 4, 7 }, { 9, 1, 4, 7, 3, 8, 2, 5, 6 }, { 7, 5, 2, 1, 4, 6, 9, 8, 3 }, { 2, 4, 3, 5, 6, 7, 8, 9, 1 }, { 5, 8, 7, 2, 1, 9, 6, 3, 4 }, { 1, 9, 6, 3, 8, 4, 5, 7, 2 }, { 8, 7, 1, 4, 9, 2, 3, 6, 5 }, { 4, 2, 9, 6, 5, 3, 7, 1, 8 }, { 3, 6, 5, 8, 7, 1, 4, 2, 9 } });

		//hard
		inputs.add("9\n. . . 9 6 . . . .\n. . . . . 8 7 . .\n. . 7 5 . . . 3 4\n. . . . 2 4 . . .\n. 8 2 . . . 3 7 .\n. . . 7 3 . . . .\n9 7 . . . 3 6 . .\n. . 5 2 . . . . .\n. . . . 7 1 . . .\n1 1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n1 1 1 2 2 2 3 3 3\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n4 4 4 5 5 5 6 6 6\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9\n7 7 7 8 8 8 9 9 9");
		solutions.add(new Integer[][] { { 3, 2, 4, 9, 6, 7, 8, 5, 1 }, { 5, 1, 9, 3, 4, 8, 7, 2, 6 }, { 8, 6, 7, 5, 1, 2, 9, 3, 4 }, { 7, 5, 3, 8, 2, 4, 1, 6, 9 }, { 4, 8, 2, 1, 9, 6, 3, 7, 5 }, { 1, 9, 6, 7, 3, 5, 2, 4, 8 }, { 9, 7, 1, 4, 5, 3, 6, 8, 2 }, { 6, 3, 5, 2, 8, 9, 4, 1, 7 }, { 2, 4, 8, 6, 7, 1, 5, 9, 3 } });
		
		//hard
		inputs.add("9\n. 4 . . 9 1 6 . .\n2 . . . . . . 5 .\n. . 6 . . . . . 3\n. 7 . 3 . . . . .\n. . 2 . 7 . 4 . .\n. . . . . 2 . 6 .\n4 . . . . . 1 . .\n. 9 . . . . . . 8\n. . 7 5 6 . . 9 .");
		solutions.add(new Integer[][]{{3,4,5,7,9,1,6,8,2},{2,8,9,6,4,3,7,5,1},{7,1,6,8,2,5,9,4,3},{5,7,4,3,8,6,2,1,9},{8,6,2,1,7,9,4,3,5},{9,3,1,4,5,2,8,6,7},{4,5,8,9,3,7,1,2,6},{6,9,3,2,1,4,5,7,8},{1,2,7,5,6,8,3,9,4}});
		
		//hard
		inputs.add("9\n. . 8 . . . . . 6\n. . . . . 2 . . .\n2 . . . 3 5 7 . .\n. . . . . . 3 4 .\n7 . . . . . . . 5\n. 9 6 . . . . . .\n. . 4 6 8 . . . 9\n. . . 7 . . . . .\n3 . . . . . 2 . .");
		solutions.add(new Integer[][]{{5,3,8,4,7,1,9,2,6},{9,4,7,8,6,2,1,5,3},{2,6,1,9,3,5,7,8,4},{8,5,2,1,9,6,3,4,7},{7,1,3,2,4,8,6,9,5},{4,9,6,3,5,7,8,1,2},{1,2,4,6,8,3,5,7,9},{6,8,5,7,2,9,4,3,1},{3,7,9,5,1,4,2,6,8}});
		
		//hard
		inputs.add("9\n. 3 4 5 . . 2 . .\n. . . . . 8 . . 6\n. . . . 2 . . . 7\n. 2 . 6 . . 1 . 8\n. . . . 4 . . . .\n3 . 7 . . 5 . 6 .\n5 . . . 7 . . . .\n9 . . 8 . . . . .\n. . 3 . . 4 5 2 .");
		solutions.add(new Integer[][]{{7,3,4,5,9,6,2,8,1},{2,5,9,7,1,8,3,4,6},{1,8,6,4,2,3,9,5,7},{4,2,5,6,3,7,1,9,8},{6,9,8,2,4,1,7,3,5},{3,1,7,9,8,5,4,6,2},{5,6,2,3,7,9,8,1,4},{9,4,1,8,5,2,6,7,3},{8,7,3,1,6,4,5,2,9}});
		
		//medium 20
		inputs.add("9\n1 . . 4 . 5 . . .\n. 6 . . 3 . . 5 .\n. . 2 . 9 . 4 . .\n4 8 . . . 2 . . 3\n. . . . . . . . .\n5 . . 9 . . . 2 1\n. . 1 . 6 . 3 . .\n. 2 . . 8 . . 9 .\n. . . 3 . 4 . . 8");
		solutions.add(new Integer[][]{{1,7,8,4,2,5,9,3,6},{9,6,4,8,3,7,1,5,2},{3,5,2,1,9,6,4,8,7},{4,8,9,7,1,2,5,6,3},{2,1,7,6,5,3,8,4,9},{5,3,6,9,4,8,7,2,1},{8,4,1,2,6,9,3,7,5},{7,2,3,5,8,1,6,9,4},{6,9,5,3,7,4,2,1,8}});
		
		//medium 30
		inputs.add("9\n. 6 2 . . . . 8 .\n. . . 3 . . . . 9\n5 . . . 2 . . . 1\n. 3 9 8 . . . . 4\n. . . . 1 . . . .\n1 . . . . 5 6 7 .\n9 . . . 8 . . . 2\n4 . . . . 3 . . .\n. 7 . . . . 4 3 .");
		solutions.add(new Integer[][]{{3,6,2,1,4,9,5,8,7},{8,4,1,3,5,7,2,6,9},{5,9,7,6,2,8,3,4,1},{7,3,9,8,6,2,1,5,4},{6,8,5,7,1,4,9,2,3},{1,2,4,9,3,5,6,7,8},{9,5,3,4,8,6,7,1,2},{4,1,6,2,7,3,8,9,5},{2,7,8,5,9,1,4,3,6}});
		
		//4x4 board
		inputs.add("4\n. 2 . 4\n3 4 . .\n. 1 . 3\n4 . . 1");
		solutions.add(new Integer[][]{{1,2,3,4},{3,4,1,2},{2,1,4,3},{4,3,2,1}});

		for (int i = 0; i < inputs.size(); ++i) {
			String s = inputs.get(i);
			Integer sol[][] = solutions.get(i);
			Board b = new Board(new StringReader(s));
			long start = System.currentTimeMillis();
			b.solve();
			System.out.println(i + " solve time: " + (System.currentTimeMillis() - start) + "ms");
			System.out.println(i + " iterations: " + b.iterations);
			for (int row = 0; row < b.size; ++row) {
				for (int col = 0; col < b.size; ++col) {
					Cell c = b.cells[row][col];
					if (c.value != sol[row][col]){
						System.out.println(b);
						System.out.println(b.toStringHTML());
						Assert.fail();
					}
				}
			}
			System.out.println();
		}
	}
}
